﻿
namespace Masterpiece.Controls.Ribbon2010.Primitives
{
   public class RibbonRepeatButton:Masterpiece.Controls.Ribbon2010.Primitives.RibbonButtonBase
   {
        #region Constructors

        static RibbonRepeatButton() 
        {
            System.Type ownerType = typeof(RibbonRepeatButton);
            RibbonRepeatButton.DefaultStyleKeyProperty.OverrideMetadata(ownerType, new System.Windows.FrameworkPropertyMetadata(ownerType));
            RibbonRepeatButton.ClickModeProperty.OverrideMetadata
                (
                ownerType,
                new System.Windows.FrameworkPropertyMetadata(System.Windows.Controls.ClickMode.Press)
                );
        }

        public RibbonRepeatButton():base() 
        {
        }

        #endregion

        #region Properties and Depenency Properties

        /// <summary>
        /// Gets or sets the amount of time, in milliseconds, the RepeatButton waits while it is pressed before it starts repeating. The value must be non-negative. This is a dependency property. 
        /// </summary>
        [System.ComponentModel.Bindable(true), System.ComponentModel.Category("Behavior")]
        public int Delay
        {
            get { return (int)GetValue(DelayProperty); }
            set { SetValue(DelayProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Delay.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty DelayProperty =
            System.Windows.DependencyProperty.Register("Delay", typeof(int), typeof(RibbonRepeatButton),
            new System.Windows.FrameworkPropertyMetadata(
                RibbonRepeatButton.GetKeyboardDelay()), new System.Windows.ValidateValueCallback(RibbonRepeatButton.IsValidDelay));

        /// <summary>
        /// Gets or sets the amount of time, in milliseconds, between repeats once repeating starts. The value must be non-negative. This is a dependency property.
        /// </summary>
        [System.ComponentModel.Bindable(true), System.ComponentModel.Category("Behavior")]
        public int Interval
        {
            get { return (int)GetValue(IntervalProperty); }
            set { SetValue(IntervalProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Interval.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty IntervalProperty =
            System.Windows.DependencyProperty.Register("Interval", typeof(int), 
            typeof(RibbonRepeatButton),
            new System.Windows.FrameworkPropertyMetadata(RibbonRepeatButton.GetKeyboardSpeed()), new System.Windows.ValidateValueCallback(RibbonRepeatButton.lsIntervalValid));

        public int ClickCount
        {
            get { return (int)GetValue(ClickCountProperty); }
            set { SetValue(ClickCountProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ClickCount.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty ClickCountProperty =
            System.Windows.DependencyProperty.Register("ClickCount", typeof(int), typeof(RibbonRepeatButton), new System.Windows.PropertyMetadata(0));

        #endregion

        #region Depenency Property Changed Callbacks

        #endregion

        #region Overrides

        protected override void OnMouseLeftButtonDown(System.Windows.Input.MouseButtonEventArgs e)
        {
            base.OnMouseLeftButtonDown(e);
            if (this.IsPressed && this.ClickMode != System.Windows.Controls.ClickMode.Hover) 
            {
                this.StartTimer();
            }
        }

        protected override void OnMouseLeftButtonUp(System.Windows.Input.MouseButtonEventArgs e)
        {
            base.OnMouseLeftButtonUp(e);

            if (ClickMode != System.Windows.Controls.ClickMode.Hover)
            {
                this.StopTimer();
            }
        }

        protected override void OnLostMouseCapture(System.Windows.Input.MouseEventArgs e)
        {
            base.OnLostMouseCapture(e);
            this.StopTimer();
        }

        protected override void OnMouseEnter(System.Windows.Input.MouseEventArgs e)
        {
            base.OnMouseEnter(e);
            if (this.HandleIsMouseOverChanged())
            {
                e.Handled = true;
            }
        }

        protected override void OnMouseLeave(System.Windows.Input.MouseEventArgs e)
        {
            base.OnMouseLeave(e);
            if (this.HandleIsMouseOverChanged())
            {
                e.Handled = true;
            }
        }

        private bool HandleIsMouseOverChanged()
        {
            if (ClickMode == System.Windows.Controls.ClickMode.Hover)
            {
                if (this.IsMouseOver)
                {
                    this.StartTimer();
                }
                else
                {
                    this.StopTimer();
                }

                return true;
            }

            return false;
        }

        protected override void OnKeyDown(System.Windows.Input.KeyEventArgs e)
        {
            base.OnKeyDown(e);
            if ((e.Key == System.Windows.Input.Key.Space) && (ClickMode != System.Windows.Controls.ClickMode.Hover))
            {
               this.StartTimer();
            }
        }

        /// <summary>
        /// This is the method that responds to the KeyUp event.
        /// </summary>
        /// <param name="e">
        protected override void OnKeyUp(System.Windows.Input.KeyEventArgs e)
        {
            if ((e.Key == System.Windows.Input.Key.Space) && (ClickMode != System.Windows.Controls.ClickMode.Hover))
            {
                this.StopTimer();
            }
            base.OnKeyUp(e);
        }
  

        #endregion

        #region Private Utilities

        private void StartTimer() 
        {
            if (this.timer == null) 
            {
                this.timer = new System.Windows.Threading.DispatcherTimer();
                this.timer.Tick += new System.EventHandler(this.OnTimeOut);
            }
            else if (this.timer.IsEnabled) 
            {
                return;
            }
            this.timer.Interval = System.TimeSpan.FromMilliseconds(this.Delay);
            this.timer.Start();
        }

        private void StopTimer() 
        {
            if (this.timer != null) 
            {
                this.timer.Stop();
            }
        }

        private static bool IsValidDelay(object value)
        {
            return (int)value>=0;
        }

        private static bool lsIntervalValid(object value) 
        {
            return (int)value > 0;
        }

        internal static int GetKeyboardDelay() 
        {
            int delay = System.Windows.SystemParameters.KeyboardDelay;
            // SPI_GETKEYBOARDDELAY 0,1,2,3 correspond to 250,500,750,1000ms
            if (delay < 0 || delay > 3)
                delay = 0;
            return (delay + 1) * 250;
        }

        internal static int GetKeyboardSpeed()
        {
            int speed = System.Windows.SystemParameters.KeyboardSpeed;
            // SPI_GETKEYBOARDSPEED 0,...,31 correspond to 1000/2.5=400,...,1000/30 ms
            if (speed < 0 || speed > 31)
                speed = 31;
            return (31 - speed) * (400 - 1000 / 30) / 31 + 1000 / 30;
        }

        #endregion

        #region Event Handlers

        private void OnTimeOut(object sender, System.EventArgs e) 
        {
            System.TimeSpan interval = System.TimeSpan.FromMilliseconds(this.Interval);
            if (this.timer.Interval != interval) 
            {
                this.timer.Interval = interval;
            }
            if (this.IsPressed) 
            {
                this.counter++;
                this.SetValue(RibbonRepeatButton.ClickCountProperty, counter);
                this.OnClick();
            }
        }

        #endregion

        #region Fields
        private int counter = 0;
        private System.Windows.Threading.DispatcherTimer timer;
        #endregion

   }
   
}
